@@ -542,7 +542,7 @@ namespace wayland
         \return A new event queue associated with this display or NULL
         on failure.
     */
-    event_queue_t create_queue();
+    event_queue_t create_queue() const;
 
     /** \brief Get a display context's file descriptor.
         \return Display object file descriptor
@@ -559,7 +559,7 @@ namespace wayland
         Blocks until the server process all currently issued requests
         and sends out pending events on all event queues.
     */
-    int roundtrip();
+    int roundtrip() const;
 
     /** \brief Block until all pending request are processed by the server.
         \return The number of dispatched events
@@ -573,7 +573,7 @@ namespace wayland
         that calling roundtrip_queue() doesn't interfere with calling
         prepare_read() and read_events())
     */
-    int roundtrip_queue(const event_queue_t& queue);
+    int roundtrip_queue(const event_queue_t& queue) const;
 
     /** \brief Announce calling thread's intention to read events from the
      * Wayland display file descriptor
@@ -607,7 +607,7 @@ namespace wayland
      * \return New \ref read_intent for this display and the default event queue
      * \exception std::system_error on failure
      */
-    read_intent obtain_read_intent();
+    read_intent obtain_read_intent() const;
 
     /** \brief Announce calling thread's intention to read events from the
      * Wayland display file descriptor
@@ -618,7 +618,7 @@ namespace wayland
      *
      * See \ref obtain_read_intent for details.
      */
-    read_intent obtain_queue_read_intent(const event_queue_t& queue);
+    read_intent obtain_queue_read_intent(const event_queue_t& queue) const;
 
     /** \brief Dispatch events in an event queue.
         \param queue The event queue to dispatch
@@ -634,7 +634,7 @@ namespace wayland
         the display fd. For other threads this will block until the main
         thread queues events on the queue passed as argument.
     */
-    int dispatch_queue(const event_queue_t& queue);
+    int dispatch_queue(const event_queue_t& queue) const;
 
     /** \brief Dispatch pending events in an event queue.
         \param queue The event queue to dispatch
@@ -646,7 +646,7 @@ namespace wayland
         appropriately. If there are no events queued, this function
         returns immediately.
     */
-    int dispatch_queue_pending(const event_queue_t& queue);
+    int dispatch_queue_pending(const event_queue_t& queue) const;
 
     /** \brief Process incoming events.
         \return The number of dispatched events
@@ -668,7 +668,7 @@ namespace wayland
         See also: display_t::dispatch_pending(),
         display_t::dispatch_queue()
     */
-    int dispatch();
+    int dispatch() const;
 
     /** \brief Dispatch main queue events without reading from the display fd.
         \return The number of dispatched events
@@ -706,7 +706,7 @@ namespace wayland
         See also: display_t::dispatch(), display_t::dispatch_queue(),
         display_t::flush()
     */
-    int dispatch_pending();
+    int dispatch_pending() const;
 
     /** \brief Retrieve the last error that occurred on a display.
         \return The last error that occurred on display or 0 if no error
@@ -734,7 +734,7 @@ namespace wayland
         in the returned tuple will be set to false. In that case, use poll on the
         display file descriptor to wait for it to become writable again.
     */
-    std::tuple<int, bool> flush();
+    std::tuple<int, bool> flush() const;
 
     /** \brief asynchronous roundtrip
 
