@@ -64,7 +64,7 @@ struct argument_t : public element_t
     throw std::runtime_error("Enum type must be int or uint");
   }
 
-  std::string print_type() const
+  std::string print_type(bool server) const
   {
     if(!interface.empty())
       return interface + "_t";
@@ -79,9 +79,9 @@ struct argument_t : public element_t
     if(type == "string")
       return "std::string";
     if(type == "object")
-      return "proxy_t";
+      return server ? "resource_t" : "proxy_t";
     if(type == "new_id")
-      return "proxy_t";
+      return server ? "resource_t" : "proxy_t";
     if(type == "fd")
       return "int";
     if(type == "array")
@@ -110,9 +110,9 @@ struct argument_t : public element_t
     return "x";
   }
 
-  std::string print_argument() const
+  std::string print_argument(bool server) const
   {
-    return print_type() + (!interface.empty() || !enum_iface.empty() || type == "string" || type == "array" ? " const& " : " ") + sanitise(name);
+    return print_type(server) + (!interface.empty() || !enum_iface.empty() || type == "string" || type == "array" ? " const& " : " ") + sanitise(name);
   }
 };
 
@@ -120,13 +120,15 @@ struct event_t : public element_t
 {
   std::list<argument_t> args;
   int since = 0;
+  argument_t ret;
+  int opcode = 0;
 
-  std::string print_functional() const
+  std::string print_functional(bool server) const
   {
     std::stringstream ss;
     ss << "    std::function<void(";
     for(auto const& arg : args)
-      ss << arg.print_type() << ", ";
+      ss << arg.print_type(server) << ", ";
     if(!args.empty())
       ss.str(ss.str().substr(0, ss.str().size()-2));
     ss.seekp(0, std::ios_base::end);
@@ -134,7 +136,7 @@ struct event_t : public element_t
     return ss.str();
   }
 
-  std::string print_dispatcher(int opcode) const
+  std::string print_dispatcher(int opcode, bool server) const
   {
     std::stringstream ss;
     ss << "    case " << opcode << ":" << std::endl
@@ -143,11 +145,16 @@ struct event_t : public element_t
     int c = 0;
     for(auto const& arg : args)
       if(!arg.enum_name.empty() && arg.type != "array")
-        ss << arg.print_type() << "(args[" << c++ << "].get<" << arg.print_enum_wire_type() << ">()), ";
+        ss << arg.print_type(server) << "(args[" << c++ << "].get<" << arg.print_enum_wire_type() << ">()), ";
       else if(!arg.interface.empty())
-        ss << arg.print_type() << "(args[" << c++ << "].get<proxy_t>()), ";
+      {
+        if(server)
+          ss << arg.print_type(server) << "(args[" << c++ << "].get<resource_t>()), ";
+        else
+          ss << arg.print_type(server) << "(args[" << c++ << "].get<proxy_t>()), ";
+      }
       else
-        ss << "args[" << c++ << "].get<" << arg.print_type() << ">(), ";
+        ss << "args[" << c++ << "].get<" << arg.print_type(server) << ">(), ";
     if(!args.empty())
       ss.str(ss.str().substr(0, ss.str().size()-2));
     ss.seekp(0, std::ios_base::end);
@@ -156,7 +163,7 @@ struct event_t : public element_t
     return ss.str();
   }
 
-  std::string print_signal_header() const
+  std::string print_signal_header(bool server) const
   {
     std::stringstream ss;
     ss << "  /** \\brief " << summary << std::endl;
@@ -167,7 +174,7 @@ struct event_t : public element_t
 
     ss << "  std::function<void(";
     for(auto const& arg : args)
-      ss << arg.print_type() + ", ";
+      ss << arg.print_type(server) + ", ";
     if(!args.empty())
       ss.str(ss.str().substr(0, ss.str().size()-2));
     ss.seekp(0, std::ios_base::end);
@@ -175,12 +182,12 @@ struct event_t : public element_t
     return ss.str();
   }
 
-  std::string print_signal_body(const std::string& interface_name) const
+  std::string print_signal_body(const std::string& interface_name, bool server) const
   {
     std::stringstream ss;
     ss << "std::function<void(";
     for(auto const& arg : args)
-      ss << arg.print_type() << ", ";
+      ss << arg.print_type(server) << ", ";
     if(!args.empty())
       ss.str(ss.str().substr(0, ss.str().size()-2));
     ss.seekp(0, std::ios_base::end);
@@ -190,12 +197,6 @@ struct event_t : public element_t
        << "}" << std::endl;
     return ss.str();
   }
-};
-
-struct request_t : public event_t
-{
-  argument_t ret;
-  int opcode = 0;
 
   std::string availability_function_name() const
   {
@@ -209,7 +210,7 @@ struct request_t : public event_t
     return name + "_since_version";
   }
 
-  std::string print_header() const
+  std::string print_header(bool server) const
   {
     std::stringstream ss;
     ss << "  /** \\brief " << summary << std::endl;
@@ -229,10 +230,10 @@ struct request_t : public event_t
     ss << description << std::endl
        << "  */" << std::endl;
 
-    if(ret.name.empty())
+    if(ret.name.empty() || server)
       ss << "  void ";
     else
-      ss << "  " << ret.print_type() << " ";
+      ss << "  " << ret.print_type(server) << " ";
     ss << sanitise(name) << "(";
 
     for(auto const& arg : args)
@@ -242,7 +243,10 @@ struct request_t : public event_t
           ss << "proxy_t &interface, uint32_t version, ";
       }
       else
-        ss << arg.print_argument() << ", ";
+        ss << arg.print_argument(server) << ", ";
+
+    if(server)
+      ss << "bool post = true";
 
     if(ss.str().substr(ss.str().size()-2, 2) == ", ")
       ss.str(ss.str().substr(0, ss.str().size()-2));
@@ -266,13 +270,13 @@ struct request_t : public event_t
     return ss.str();
   }
 
-  std::string print_body(const std::string& interface_name) const
+  std::string print_body(const std::string& interface_name, bool server) const
   {
     std::stringstream ss;
-    if(ret.name.empty())
+    if(ret.name.empty() || server)
       ss <<  "void ";
     else
-      ss << ret.print_type() << " ";
+      ss << ret.print_type(server) << " ";
     ss << interface_name << "_t::" << sanitise(name) << "(";
 
     bool new_id_arg = false;
@@ -286,14 +290,20 @@ struct request_t : public event_t
         }
       }
       else
-        ss << arg.print_argument() << ", ";
+        ss << arg.print_argument(server) << ", ";
+
+    if(server)
+      ss << "bool post";
 
     if(ss.str().substr(ss.str().size()-2, 2) == ", ")
       ss.str(ss.str().substr(0, ss.str().size()-2));
     ss.seekp(0, std::ios_base::end);
-    ss << ")\n{" << std::endl;
+    ss << ")" << std::endl
+       << "{" << std::endl;
 
-    if(ret.name.empty())
+    if(server)
+      ss <<  "  send_event(post, " << opcode << ", ";
+    else if(ret.name.empty())
       ss <<  "  marshal(" << opcode << "U, ";
     else if(ret.interface.empty())
     {
@@ -326,7 +336,7 @@ struct request_t : public event_t
     ss.seekp(0, std::ios_base::end);
     ss << ");" << std::endl;
 
-    if(!ret.name.empty())
+    if(!ret.name.empty() && !server)
     {
       if(new_id_arg)
       {
@@ -334,9 +344,9 @@ struct request_t : public event_t
            << "  return interface;" << std::endl;
       }
       else
-        ss << "  return " << ret.print_type() << "(p);" << std::endl;
+        ss << "  return " << ret.print_type(server) << "(p);" << std::endl;
     }
-    ss << "}";
+    ss << "}" << std::endl;
 
     if(!availability_function_name().empty())
     {
@@ -344,13 +354,17 @@ struct request_t : public event_t
          << "bool " << interface_name << "_t::" << availability_function_name() << "() const" << std::endl
          << "{" << std::endl
          << "  return (get_version() >= " << since_version_constant_name() << ");" << std::endl
-         << "}";
+         << "}" << std::endl;
     }
 
     return ss.str();
   }
 };
 
+struct request_t : public event_t
+{
+};
+
 struct enum_entry_t : public element_t
 {
   std::string value;
@@ -384,12 +398,12 @@ struct enumeration_t : public element_t
       ss << "enum class " << iface_name << "_" << name << " : uint32_t" << std::endl
          << "  {" << std::endl;
     else
-      ss << "struct " << iface_name << "_" << name << " : public detail::bitfield<" << width << ", " << id << ">" << std::endl
+      ss << "struct " << iface_name << "_" << name << " : public wayland::detail::bitfield<" << width << ", " << id << ">" << std::endl
          << "{" << std::endl
-         << "  " << iface_name << "_" << name << "(const detail::bitfield<" << width << ", " << id << "> &b)" << std::endl
-         << "    : detail::bitfield<" << width << ", " << id << ">(b) {}" << std::endl
+         << "  " << iface_name << "_" << name << "(const wayland::detail::bitfield<" << width << ", " << id << "> &b)" << std::endl
+         << "    : wayland::detail::bitfield<" << width << ", " << id << ">(b) {}" << std::endl
          << "  " << iface_name << "_" << name << "(const uint32_t value)" << std::endl
-         << "    : detail::bitfield<" << width << ", " << id << ">(value) {}" << std::endl;
+         << "    : wayland::detail::bitfield<" << width << ", " << id << ">(value) {}" << std::endl;
 
     for(auto const& entry : entries)
     {
@@ -399,7 +413,7 @@ struct enumeration_t : public element_t
       if(!bitfield)
         ss << "  " << sanitise(entry.name) << " = " << entry.value << "," << std::endl;
       else
-        ss << "  static const detail::bitfield<" << width << ", " << id << "> " << sanitise(entry.name) << ";" << std::endl;
+        ss << "  static const wayland::detail::bitfield<" << width << ", " << id << "> " << sanitise(entry.name) << ";" << std::endl;
     }
 
     if(!bitfield)
@@ -426,6 +440,30 @@ struct enumeration_t : public element_t
   }
 };
 
+struct post_error_t : public element_t
+{
+  std::string print_server_header() const
+  {
+    std::stringstream ss;
+    ss << "  /** \\brief Post error: " << summary << std::endl;
+    if(!description.empty())
+      ss << "  " << description << std::endl;
+    ss << "  */" << std::endl;
+    ss << "  void post_" << name << "(std::string const& msg);" << std::endl;
+    return ss.str();
+  }
+
+  std::string print_server_body(std::string iface_name) const
+  {
+    std::stringstream ss;
+    ss << "void " << iface_name << "_t::post_" << name << "(std::string const& msg)" << std::endl
+       << "{" << std::endl
+       << "  post_error(static_cast<uint32_t>(" << iface_name << "_error::" << sanitise(name) << "), msg);" << std::endl
+       << "}" << std::endl;
+    return ss.str();
+  }
+};
+
 struct interface_t : public element_t
 {
   int version = 0;
@@ -434,6 +472,7 @@ struct interface_t : public element_t
   std::list<request_t> requests;
   std::list<event_t> events;
   std::list<enumeration_t> enums;
+  std::list<post_error_t> errors;
 
   std::string print_forward() const
   {
@@ -451,7 +490,7 @@ struct interface_t : public element_t
     return ss.str();
   }
 
-  std::string print_header() const
+  std::string print_client_header() const
   {
     std::stringstream ss;
     ss << "/** \\brief " << summary << std::endl
@@ -465,7 +504,7 @@ struct interface_t : public element_t
        << "  {" << std::endl;
 
     for(auto const& event : events)
-      ss << event.print_functional() << std::endl;
+      ss << event.print_functional(false) << std::endl;
 
     ss << "  };" << std::endl
        << std::endl
@@ -488,10 +527,10 @@ struct interface_t : public element_t
 
     for(auto const& request : requests)
       if(request.name != "destroy")
-        ss << request.print_header() << std::endl;
+        ss << request.print_header(false) << std::endl;
 
     for(auto const& event : events)
-      ss << event.print_signal_header() << std::endl;
+      ss << event.print_signal_header(false) << std::endl;
 
     ss << "};" << std::endl
        << std::endl;
@@ -502,6 +541,65 @@ struct interface_t : public element_t
     return ss.str();
   }
 
+  std::string print_server_header() const
+  {
+    std::stringstream ss;
+    ss << "/** \\brief " << summary << std::endl
+       << description << std::endl
+       << "*/" << std::endl;
+
+    ss << "class " << name << "_t : public resource_t" << std::endl
+       << "{" << std::endl
+       << "private:" << std::endl
+       << "  struct events_t : public resource_t::events_base_t" << std::endl
+       << "  {" << std::endl;
+
+    for(auto const& request : requests)
+      ss << request.print_functional(true) << std::endl;
+
+    ss << "  };" << std::endl
+       << std::endl
+       << "  static int dispatcher(int opcode, const std::vector<wayland::detail::any>& args, const std::shared_ptr<resource_t::events_base_t>& e);" << std::endl
+       << std::endl;
+
+    ss << "protected:" << std::endl
+       << "  static constexpr const wl_interface *interface = &wayland::server::detail::" << name << "_interface;" << std::endl
+       << "  static constexpr const unsigned int max_version = " << version << ";" << std::endl
+       << std::endl
+       << "  friend class global_t<" << name << "_t>;" << std::endl
+       << "  friend class global_base_t;" << std::endl
+       << std::endl;
+
+    ss << "public:" << std::endl
+       << "  " << name << "_t() = default;" << std::endl
+       << "  " << name << "_t(const client_t& client, uint32_t id, int version = " << version << ");" << std::endl
+       << "  " << name << "_t(const resource_t &resource);" << std::endl
+       << std::endl
+       << "  static const std::string interface_name;" << std::endl
+       << std::endl
+       << "  operator " << orig_name << "*() const;" << std::endl
+       << std::endl;
+
+    for(auto const& request : requests)
+      ss << request.print_signal_header(true) << std::endl;
+
+    for(auto const& event : events)
+      ss << event.print_header(true) << std::endl;
+
+    for(auto const& error : errors)
+      ss << error.print_server_header() << std::endl;
+
+    ss << "};" << std::endl
+       << std::endl
+       << "using global_" << name << "_t = global_t<" << name << "_t>;" << std::endl
+       << std::endl;
+
+    for(auto const& enumeration : enums)
+      ss << enumeration.print_header(name) << std::endl;
+
+    return ss.str();
+  }
+
   std::string print_interface_header() const
   {
     std::stringstream ss;
@@ -509,7 +607,7 @@ struct interface_t : public element_t
     return ss.str();
   }
 
-  std::string print_body() const
+  std::string print_client_body() const
   {
     std::stringstream set_events;
     set_events << "  if(proxy_has_object() && get_wrapper_type() == wrapper_type::standard)" << std::endl
@@ -565,11 +663,11 @@ struct interface_t : public element_t
 
     for(auto const& request : requests)
       if(request.name != "destroy")
-        ss << request.print_body(name) << std::endl
+        ss << request.print_body(name, false) << std::endl
            << std::endl;
 
     for(auto const& event : events)
-      ss << event.print_signal_body(name) << std::endl;
+      ss << event.print_signal_body(name, false) << std::endl;
 
     ss << "int " << name << "_t::dispatcher(uint32_t opcode, const std::vector<any>& args, const std::shared_ptr<detail::events_base_t>& e)" << std::endl
        << "{" << std::endl;
@@ -582,7 +680,7 @@ struct interface_t : public element_t
 
       int opcode = 0;
       for(auto const& event : events)
-        ss << event.print_dispatcher(opcode++) << std::endl;
+        ss << event.print_dispatcher(opcode++, false) << std::endl;
 
       ss << "    }" << std::endl;
     }
@@ -596,12 +694,70 @@ struct interface_t : public element_t
     return ss.str();
   }
 
-  std::string print_interface_body() const
+  std::string print_server_body() const
+  {
+    std::stringstream ss;
+    ss << name << "_t::" << name << "_t(const client_t& client, uint32_t id, int version)" << std::endl
+       << "  : resource_t(client, &server::detail::" << name << "_interface, id, version)" << std::endl
+       << "{" << std::endl
+       << "  set_events(std::shared_ptr<resource_t::events_base_t>(new events_t), dispatcher);" << std::endl
+       << "}" << std::endl
+       << std::endl
+       << name << "_t::" << name << "_t(const resource_t &resource)" << std::endl
+       << "  : resource_t(resource)" << std::endl
+       << "{" << std::endl
+       << "  set_events(std::shared_ptr<resource_t::events_base_t>(new events_t), dispatcher);" << std::endl
+       << "}" << std::endl
+       << std::endl
+       << "const std::string " << name << "_t::interface_name = \"" << orig_name << "\";" << std::endl
+       << std::endl
+       << name << "_t::operator " << orig_name << "*() const" << std::endl
+       << "{" << std::endl
+       << "  return reinterpret_cast<" << orig_name << "*> (c_ptr());" << std::endl
+       << "}" << std::endl
+       << std::endl;
+
+    for(auto const& request : requests)
+      ss << request.print_signal_body(name, true) << std::endl
+         << std::endl;
+
+    for(auto const& event : events)
+      ss << event.print_body(name, true) << std::endl;
+
+    for(auto const& error : errors)
+      ss << error.print_server_body(name) << std::endl;
+
+    ss << "int " << name << "_t::dispatcher(int opcode, const std::vector<any>& args, const std::shared_ptr<resource_t::events_base_t>& e)" << std::endl
+       << "{" << std::endl;
+
+    if(!requests.empty())
+    {
+      ss << "  std::shared_ptr<events_t> events = std::static_pointer_cast<events_t>(e);" << std::endl
+         << "  switch(opcode)" << std::endl
+         << "    {" << std::endl;
+
+      int opcode = 0;
+      for(auto const& request : requests)
+        ss << request.print_dispatcher(opcode++, true) << std::endl;
+
+      ss << "    }" << std::endl;
+    }
+
+    ss << "  return 0;" << std::endl
+       << "}" << std::endl;
+
+    for(auto const& enumeration : enums)
+      ss << enumeration.print_body(name) << std::endl;
+
+    return ss.str();
+  }
+
+  std::string print_interface_body(bool server) const
   {
     std::stringstream ss;
     for(auto const& request : requests)
     {
-      ss << "const wl_interface* " << name << "_interface_" << request.name << "_request[" << request.args.size() << "] = {" << std::endl;
+      ss << "const wl_interface* " << name << "_interface_" << request.name << "_request" << (server ? "_server" : "") << "[" << request.args.size() << "] = {" << std::endl;
       for(auto const& arg : request.args)
         if(!arg.interface.empty())
           ss  << "  &" << arg.interface << "_interface," << std::endl;
@@ -612,7 +768,7 @@ struct interface_t : public element_t
     }
     for(auto const& event : events)
     {
-      ss << "const wl_interface* " << name << "_interface_" << event.name << "_event[" << event.args.size() << "] = {" << std::endl;
+      ss << "const wl_interface* " << name << "_interface_" << event.name << "_event" << (server ? "_server" : "") << "[" << event.args.size() << "] = {" << std::endl;
       for(auto const& arg : event.args)
         if(!arg.interface.empty())
           ss  << "  &" << arg.interface << "_interface," << std::endl;
@@ -621,7 +777,7 @@ struct interface_t : public element_t
       ss << "};" << std::endl
          << std::endl;
     }
-    ss << "const wl_message " << name << "_interface_requests[" << requests.size() << "] = {" << std::endl;
+    ss << "const wl_message " << name << "_interface_requests" << (server ? "_server" : "") << "[" << requests.size() << "] = {" << std::endl;
     for(auto const& request : requests)
     {
       ss << "  {" << std::endl
@@ -638,12 +794,12 @@ struct interface_t : public element_t
         ss << arg.print_short();
       }
       ss << "\"," << std::endl
-         << "    " << name << "_interface_" << request.name << "_request," << std::endl
+         << "    " << name << "_interface_" << request.name << "_request" << (server ? "_server" : "") << "," << std::endl
          << "  }," << std::endl;
     }
     ss << "};" << std::endl
        << std::endl;
-    ss << "const wl_message " << name << "_interface_events[" << events.size() << "] = {" << std::endl;
+    ss << "const wl_message " << name << "_interface_events" << (server ? "_server" : "") << "[" << events.size() << "] = {" << std::endl;
     for(auto const& event : events)
     {
       ss << "  {" << std::endl
@@ -660,19 +816,22 @@ struct interface_t : public element_t
         ss << arg.print_short();
       }
       ss << "\"," << std::endl
-         << "    " << name << "_interface_" << event.name << "_event," << std::endl
+         << "    " << name << "_interface_" << event.name << "_event" << (server ? "_server" : "") << "," << std::endl
          << "  }," << std::endl;
     }
     ss << "};" << std::endl
        << std::endl;
-    ss << "const wl_interface wayland::detail::" << name << "_interface =" << std::endl
-       << "  {" << std::endl
+    if(server)
+      ss << "const wl_interface wayland::server::detail::" << name << "_interface =" << std::endl;
+    else
+      ss << "const wl_interface wayland::detail::" << name << "_interface =" << std::endl;
+    ss << "  {" << std::endl
        << "    \"" << orig_name << "\"," << std::endl
        << "    " << version << "," << std::endl
        << "    " << requests.size() << "," << std::endl
-       << "    " << name << "_interface_requests," << std::endl
+       << "    " << name << "_interface_requests" << (server ? "_server" : "") << "," << std::endl
        << "    " << events.size() << "," << std::endl
-       << "    " << name << "_interface_events," << std::endl
+       << "    " << name << "_interface_events" << (server ? "_server" : "") << "," << std::endl
        << "  };" << std::endl
        << std::endl;
 
@@ -710,6 +869,7 @@ void parse_args(int argc, char **argv, std::vector<arg_t>& map, std::vector<std:
       opts_end = true;
     else
     {
+      std::string key = str.substr(1);
       std::string value;
       if(c + 1 < argc && argv[c+1][0] != '-')
         value = argv[++c];
@@ -727,10 +887,19 @@ int main(int argc, char *argv[])
   if(extra.size() < 3)
   {
     std::cerr << "Usage:" << std::endl
-              << "  " << argv[0] << " [-x extra_header.hpp] protocol1.xml [protocol2.xml ...] protocol.hpp protocol.cpp" << std::endl;
+              << "  " << argv[0] << " [-s on] [-x extra_header.hpp] protocol1.xml [protocol2.xml ...] protocol.hpp protocol.cpp" << std::endl;
     return 1;
   }
 
+  // generate server headers?
+  auto const server = [&map] ()
+  {
+    for(auto const& opt : map)
+      if(opt.key == "s")
+        return true;
+    return false;
+  }();
+
   std::list<interface_t> interfaces;
   int enum_id = 0;
 
@@ -817,9 +986,11 @@ int main(int argc, char *argv[])
         iface.requests.push_back(req);
       }
 
+      opcode = 0;
       for(auto const& event : interface.children("event"))
       {
         event_t ev;
+        ev.opcode = opcode++;
         ev.name = event.attribute("name").value();
 
         if(event.attribute("since"))
@@ -863,6 +1034,8 @@ int main(int argc, char *argv[])
 
           arg.allow_null = argument.attribute("allow-null") && std::string(argument.attribute("allow-null").value()) == "true";
 
+          if(arg.type == "new_id")
+            ev.ret = arg;
           ev.args.push_back(arg);
         }
         iface.events.push_back(ev);
@@ -907,6 +1080,15 @@ int main(int argc, char *argv[])
             enu.width = tmp;
 
           enu.entries.push_back(enum_entry);
+          if(enu.name == "error")
+
+          {
+            post_error_t error;
+            error.name = enum_entry.name;
+            error.summary = enum_entry.summary;
+            error.description = enum_entry.description;
+            iface.errors.push_back(error);
+          }
         }
         iface.enums.push_back(enu);
       }
@@ -929,7 +1111,38 @@ int main(int argc, char *argv[])
               << "#include <string>" << std::endl
               << "#include <vector>" << std::endl
               << std::endl
-              << "#include <wayland-client.hpp>" << std::endl;
+              << (server ? "#include <wayland-server.hpp>" : "#include <wayland-client.hpp>") << std::endl;
+
+  std::fstream wayland_server_hpp;
+  std::fstream wayland_server_cpp;
+  if(server)
+  {
+    std::string server_hpp_file(extra[extra.size()-2]), server_cpp_file(extra[extra.size()-1]);
+    wayland_server_hpp.open(server_hpp_file, std::ios_base::out | std::ios_base::trunc);
+    wayland_server_cpp.open(server_cpp_file, std::ios_base::out | std::ios_base::trunc);
+
+    // header intro
+    wayland_server_hpp << "#pragma once" << std::endl
+                       << std::endl
+                       << "#include <array>" << std::endl
+                       << "#include <functional>" << std::endl
+                       << "#include <memory>" << std::endl
+                       << "#include <string>" << std::endl
+                       << "#include <vector>" << std::endl
+                       << std::endl
+                       << "#include <wayland-server.hpp>" << std::endl
+                       << std::endl;
+
+    // body intro
+    auto server_hpp_slash_pos = server_hpp_file.find_last_of('/');
+    auto server_hpp_basename = (server_hpp_slash_pos == std::string::npos ? server_hpp_file : server_hpp_file.substr(server_hpp_slash_pos + 1));
+    wayland_server_cpp << "#include <" << server_hpp_basename << ">" << std::endl
+                       << std::endl
+                       << "using namespace wayland::detail;" << std::endl
+                       << "using namespace wayland::server;" << std::endl
+                       << "using namespace wayland::server::detail;" << std::endl
+                       << std::endl;
+  }
 
   for(auto const& opt : map)
     if(opt.key == std::string("x"))
@@ -945,6 +1158,9 @@ int main(int argc, char *argv[])
 
   wayland_hpp << "namespace wayland" << std::endl
               << "{" << std::endl;
+  if(server)
+    wayland_hpp << "namespace server" << std::endl
+                << "{" << std::endl;
 
   // C++ forward declarations
   for(auto const& iface : interfaces)
@@ -963,9 +1179,16 @@ int main(int argc, char *argv[])
   // class declarations
   for(auto const& iface : interfaces)
     if(iface.name != "display")
-      wayland_hpp << iface.print_header() << std::endl;
+    {
+      if(server)
+        wayland_hpp << iface.print_server_header() << std::endl;
+      else
+        wayland_hpp << iface.print_client_header() << std::endl;
+    }
   wayland_hpp << std::endl
               << "}" << std::endl;
+  if(server)
+    wayland_hpp << "}" << std::endl;
 
   // body intro
   auto hpp_slash_pos = hpp_file.find_last_of('/');
@@ -973,17 +1196,25 @@ int main(int argc, char *argv[])
   wayland_cpp << "#include <" << hpp_basename << ">" << std::endl
               << std::endl
               << "using namespace wayland;" << std::endl
-              << "using namespace detail;" << std::endl
-              << std::endl;
+              << "using namespace wayland::detail;" << std::endl;
+  if(server)
+    wayland_cpp << "using namespace wayland::server;" << std::endl
+                << "using namespace wayland::server::detail;" << std::endl;
+  wayland_cpp << std::endl;
 
   // interface bodys
   for(auto const& iface : interfaces)
-    wayland_cpp << iface.print_interface_body();
+    wayland_cpp << iface.print_interface_body(server);
 
   // class member definitions
   for(auto const& iface : interfaces)
     if(iface.name != "display")
-      wayland_cpp << iface.print_body() << std::endl;
+    {
+      if(server)
+        wayland_cpp << iface.print_server_body() << std::endl;
+      else
+        wayland_cpp << iface.print_client_body() << std::endl;
+    }
   wayland_cpp << std::endl;
 
   // clean up
